'use strict';

/* For ESLint: List any global identifiers used in this file below */
/* global browser, BG, Fennec, translate, log */

const customizeInit = function () {
  browser.runtime.onMessage.addListener((request) => {
    if (request && request.command !== 'filters_updated') {
      return;
    }

    if (!request) {
      return;
    }

    if ($('#txtFiltersAdvanced').prop('disabled') === false) {
      return;
    }

    BG.getCustomFiltersText((customFiltersResponse) => {
      $('#txtFiltersAdvanced').val(customFiltersResponse);
    });

    BG.getExcludeFiltersText((excludeFiltersResponse) => {
      $('#txtExcludeFiltersAdvanced').val(excludeFiltersResponse);
    });
  });

  // Update custom filter count in the background.
  // Inputs: customFiltersText:string - string representation of the custom filters
  // delimited by new line.
  function updateCustomFiltersCount(customFiltersText) {
    const customFiltersArray = customFiltersText.split('\n');
    const newCount = {};
    const tempFilterTracker = [];
    for (let i = 0; i < customFiltersArray.length; i += 1) {
      const filter = customFiltersArray[i];

      // Check if filter is a duplicate and that it is a hiding filter.
      if (tempFilterTracker.indexOf(filter) < 0 && filter.indexOf('##') > -1) {
        tempFilterTracker.push(filter);
        const host = filter.split('##')[0];
        newCount[host] = (newCount[host] || 0) + 1;
      }
    }

    BG.updateCustomFilterCountMap(newCount);
  }

  function saveFilters() {
    const customFiltersText = $('#txtFiltersAdvanced').val();
    const customFiltersArray = customFiltersText.split('\n');
    $('#messagecustom').empty();
    $('#messagecustom').hide();
    const response = BG.FilterNormalizer.validateList(customFiltersArray, true, true);
    if (response && !response.exception) {
      BG.setCustomFiltersText(customFiltersText);
      updateCustomFiltersCount(customFiltersText);
      $('#divAddNewFilter').slideDown();
      $('#txtFiltersAdvanced').attr('disabled', 'disabled');
      $('#spanSaveButton').hide();
      $('#btnEditAdvancedFilters').show();
      $('#btnCleanUp').show();
    } else if (response && response.exception) {
      const filterErrorMessage = translate('customfilterserrormessage', [response.filter, response.exception]);
      $('#messagecustom').text(filterErrorMessage);
      $('#messagecustom').show();
    }
    if (Fennec) {
      $('.desktop-only').hide();
    }
  }

  // Add a custom filter to the list
  function appendCustomFilter(filter) {
    const customFilterText = $('#txtFiltersAdvanced').val();
    $('#txtFiltersAdvanced').val(`${filter}\n${customFilterText}`);
    saveFilters();
    $('.addControls').slideUp();
  }

  // Convert a messy list of domains to ~domain1.com|~domain2.com format
  function toTildePipeFormat(domainList) {
    let convertedDomainList = domainList.trim().replace(/[ ,;|]+~?/g, '|~');
    if (convertedDomainList && convertedDomainList[0] !== '~') {
      convertedDomainList = `~${convertedDomainList}`;
    }
    return convertedDomainList;
  }

  $('#txtBlacklist').focus(function blacklistFocusFunction() {
    // Find the blacklist entry in the user's filters, and put it
    // into the blacklist input.
    const customFilterText = $('#txtFiltersAdvanced').val();
    const match = customFilterText.match(/^@@\*\$document,domain=(~.*)$/m);
    if (match && $(this).val() === '') {
      $(this).val(match[1]);
    }
  });

  // The add_filter functions
  $('#btnAddUserFilter').click(function userFilterClickFunction() {
    const blockCss = $('#txtUserFilterCss').val().trim();
    const blockDomain = $('#txtUserFilterDomain').val().trim();

    if (blockDomain === '.*' || blockDomain === '*' || blockDomain === '') {
      appendCustomFilter(`##${blockCss}`);
    } else {
      appendCustomFilter(`${blockDomain}##${blockCss}`);
    }

    $(this).closest('.entryTable').find("input[type='text']").val('');
    $(this).attr('disabled', 'disabled');
  });

  $('#btnAddExcludeFilter').click(function excludeFilterClickFunction() {
    let excludeUrl = $('#txtUnblock').val().trim();

    // prevent regexes
    if (/^\/.*\/$/.test(excludeUrl)) {
      excludeUrl = `${excludeUrl}*`;
    }

    appendCustomFilter(`@@${excludeUrl}$document`);

    $(this).closest('.entryTable').find("input[type='text']").val('');
    $(this).attr('disabled', 'disabled');
  });

  $('#btnAddBlacklist').click(() => {
    const blacklist = toTildePipeFormat($('#txtBlacklist').val());

    let filters = `${$('#txtFiltersAdvanced').val().trim()}\n`;

    // Delete the first likely line
    filters = filters.replace(/^@@\*\$document,domain=~.*\n/m, '').trim();
    $('#txtFiltersAdvanced').val(filters);

    // Add our line in its place, or if it was empty, remove the filter
    if (blacklist) {
      appendCustomFilter(`@@*$document,domain=${blacklist}`);
    } else {
      saveFilters();
    } // just record the deletion
    $('#btnAddBlacklist').attr('disabled', 'disabled');
  });

  $('#btnAddUrlBlock').click(function urlBlockClickFunction() {
    let blockUrl = $('#txtBlockUrl').val().trim();
    let blockDomain = $('#txtBlockUrlDomain').val().trim();
    if (blockDomain === '*') {
      blockDomain = '';
    }

    // prevent regexes
    if (/^\/.*\/$/.test(blockUrl)) {
      blockUrl = `${blockUrl}*`;
    }

    if (blockDomain === '') {
      appendCustomFilter(blockUrl);
    } else {
      appendCustomFilter(`${blockUrl}$domain=${blockDomain}`);
    }

    $(this).closest('.entryTable').find("input[type='text']").val('');
    $(this).attr('disabled', 'disabled');
  });

  // The validation functions
  $('#txtBlacklist').bind('input', () => {
    let blacklist = toTildePipeFormat($('#txtBlacklist').val());

    if (blacklist) {
      blacklist = `@@*$document,domain=${blacklist}`;
    }
    $('#messageBlacklist').empty();
    $('#messageBlacklist').hide();
    const response = BG.FilterNormalizer.validateLine(blacklist, true, true);
    if (response && !response.exception) {
      $('#btnAddBlacklist').removeAttr('disabled');
    } else if (response && response.exception) {
      $('#btnAddBlacklist').attr('disabled', 'disabled');
      const filterErrorMessage = translate('customfilterserrormessage', [blacklist, response.exception]);
      $('#messageBlacklist').text(filterErrorMessage);
      $('#messageBlacklist').show();
    } else {
      $('#btnAddBlacklist').attr('disabled', 'disabled');
    }
  });

  $("#divUrlBlock input[type='text']").bind('input', () => {
    const blockUrl = $('#txtBlockUrl').val().trim();
    let blockDomain = $('#txtBlockUrlDomain').val().trim();
    if (blockDomain === '*') {
      blockDomain = '';
    }
    if (blockDomain) {
      blockDomain = `$domain=${blockDomain}`;
    }
    let ok = false;

    const response = BG.FilterNormalizer.validateLine((blockUrl + blockDomain));
    if (response) {
      ok = true;
    }
    const secondResponse = BG.Filter.isSelectorFilter(blockUrl);
    if (secondResponse) {
      ok = false;
    }

    $('#btnAddUrlBlock').attr('disabled', ok ? null : 'disabled');
  });

  $("#divCssBlock input[type='text']").bind('input', () => {
    const blockCss = $('#txtUserFilterCss').val().trim();
    let blockDomain = $('#txtUserFilterDomain').val().trim();
    if (blockDomain === '*') {
      blockDomain = '';
    }
    let ok = false;
    const response = BG.FilterNormalizer.validateLine(`${blockDomain}##${blockCss}`);
    if (response) {
      ok = true;
    }
    $('#btnAddUserFilter').attr('disabled', ok ? null : 'disabled');
  });

  $("#divExcludeBlock input[type='text']").bind('input', () => {
    const unblockUrl = $('#txtUnblock').val().trim();
    let ok = false;
    const response = BG.FilterNormalizer.validateLine(`@@${unblockUrl}$document`);
    if (response) {
      ok = true;
    }

    const secondResponse = BG.Filter.isSelectorFilter(unblockUrl);
    if (!unblockUrl || secondResponse) {
      ok = false;
    }

    $('#btnAddExcludeFilter').attr('disabled', ok ? null : 'disabled');
  });

  // When one presses 'Enter', pretend it was a click on the 'add' button
  $(".entryTable input[type='text']").keypress(function keypressFunction(event) {
    const submitButton = $(this).closest('.entryTable').find("input[type='button']");
    if (event.keyCode === 13 && !submitButton.prop('disabled')) {
      event.preventDefault();
      submitButton.click();
    }
  });

  $('a.controlsLink').click(function controlsLinkClickFunction(event) {
    try {
      event.preventDefault();
      const myControls = $(this).closest('li').find('.addControls');
      const otherControls = $('.addControls').not(myControls);
      otherControls.slideUp();
      // Make sure to rotate the other icons back to zero
      const otherIcons = otherControls.siblings('.controlsLink').find('i');
      otherIcons.css({ 'transform-origin': '60%', transition: 'transform .5' });
      otherIcons.css({ transform: 'none' });
      // Slide panel
      myControls.slideToggle();
      const myIcon = $(this).find('i');
      let rot = myIcon.css('transform');
      if (rot === 'none') {
        rot = 'rotate(180deg)';
      } else {
        rot = 'none';
      }
      myIcon.css({ 'transform-origin': '60%', transition: 'transform .5s' });
      myIcon.css({ transform: rot });
    } catch (e) {
      log(e);
    }
  });

  $('#btnEditAdvancedFilters').click(() => {
    $('#divAddNewFilter').slideUp();
    $('.addControls').slideUp();
    $('#txtFiltersAdvanced').removeAttr('disabled');
    $('#spanSaveButton').show();
    $('#btnEditAdvancedFilters').hide();
    $('#txtFiltersAdvanced').focus();
  });

  $('#btnEditExcludeAdvancedFilters').click(() => {
    $('#divAddNewFilter').slideUp();
    $('.addControls').slideUp();
    $('#txtExcludeFiltersAdvanced').removeAttr('disabled');
    $('#spanSaveExcludeButton').show();
    $('#btnEditExcludeAdvancedFilters').hide();
    $('#txtExcludeFiltersAdvanced').focus();
  });

  function saveExcludeFilters() {
    const excludeFiltersText = $('#txtExcludeFiltersAdvanced').val();
    BG.setExcludeFilters(excludeFiltersText);
    $('#divAddNewFilter').slideDown();
    $('#txtExcludeFiltersAdvanced').attr('disabled', 'disabled');
    $('#spanSaveExcludeButton').hide();
    $('#btnEditExcludeAdvancedFilters').show();
    BG.getExcludeFiltersText((excludeFiltersResponse) => {
      $('#txtExcludeFiltersAdvanced').val(excludeFiltersResponse);
      if (excludeFiltersResponse) {
        $('#divExcludeFilters').show();
      }
    });
  }

  $('#btnSaveExcludeAdvancedFilters').click(saveExcludeFilters);

  $('#btnSaveAdvancedFilters').click(saveFilters);
  BG.getCustomFiltersText((customFiltersResponse) => {
    $('#txtFiltersAdvanced').val(customFiltersResponse);
  });

  const settings = BG.getSettings();
  if (settings.show_advanced_options) {
    $('#divExcludeFilters').show();
  }

  BG.getExcludeFiltersText((excludeFiltersResponse) => {
    $('#txtExcludeFiltersAdvanced').val(excludeFiltersResponse);
    if (excludeFiltersResponse) {
      $('#divExcludeFilters').show();
    }
  });

  $('#btnCleanUp').click(() => {
    // Don't save immediately, first allow them to review changes
    if ($('#btnEditAdvancedFilters').is(':visible')) {
      $('#btnEditAdvancedFilters').click();
    }
    const response = BG.FilterNormalizer.normalizeList($('#txtFiltersAdvanced').val(), true, true);
    let newFilters = response;
    newFilters = newFilters.replace(/(\n)+$/, '\n'); // Del trailing \n's
    $('#txtFiltersAdvanced').val(newFilters);
  });
};
